require 'metasm'
require 'metasm/gui/gtk'
include Metasm
require '../deobfuscate.rb'
require '../analyse_handlers'
require '../macro_cpu_vm'

# graph-only view
module Metasm
module GtkGui
class DisasmWidget
attr_accessor :gui_update_counter_limit
def initialize(dasm, ep=[])
	super()
	
	@dasm = dasm
	@entrypoints = ep
	@views = []
	@pos_history = []
	@gui_update_counter_limit = 100
	
	gui_update_counter = 0
	dasm_working = false
	@gtk_idle_handle = Gtk.idle_add {
		# metasm disassembler loop
		# update gui once in a while
		dasm_working = true if not @entrypoints.empty? or not @dasm.addrs_todo.empty?
		if dasm_working
			begin
				if not @dasm.disassemble_mainiter(@entrypoints)
					dasm_working = false
					gui_update_counter = @gui_update_counter_limit
				end
			rescue
				messagebox $!
			end
			gui_update_counter += 1
			if gui_update_counter > @gui_update_counter_limit
				gui_update_counter = 0
				gui_update
			end
		end
		true
	}
	@notebook = Gtk::Notebook.new
	pack_start @notebook
	@notebook.show_border = false
	@notebook.show_tabs = false
	@views << GraphViewWidget.new(@dasm, self)
	@notebook.append_page(@views[0], Gtk::Label.new('graph'))
	@notebook.focus_child = curview
end
end
end
end

# remove the cached handlers analysis
File.unlink('saved-handlers') rescue nil

$VERBOSE = true

exe = AutoExe.decode_file('../driver.sys')
exe.cpu = Ia32_junkfilter.new	# deobfuscate ia32 code

# create the t2 vm disassembler on top of the ia32 one
t2exe = exe.dup
t2exe.cpu = T2CPU.new(exe)
t2exe.cpu.ia32dasm.cpu.showrdtsc = false
t2dasm = T2Disassembler.new(t2exe)

# the VM entrypoint
t2entry = T2Addr.new(0x13000+0xe9fe, 0x13000+0xeea6)
ep = [t2entry]

t2dasm.disassemble_mainiter ep
t2dasm.disassemble_mainiter ep
w = GtkGui::MainWindow.new.display(t2dasm, ep)
# update the gui each round
w.dasm_widget.gui_update_counter_limit = 1
w.dasm_widget.focus_addr t2entry
w.dasm_widget.views[0].set_font File.read('fontdesc')
w.signal_connect('destroy') { Gtk.main_quit }
Gtk.main
